/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pb162.timetrial;

import java.util.Date;

/**
 *
 * @author misko
 */
public interface Contestant extends Comparable<Contestant> {

    /**
     * Compares this contestant with another one. Ordering is by (time,id)
     * lexicographicly. Object with n/a time are considered as the had infinite time
     * @param o contestant to compare
     * @return -1 if the this contestant is better than the compared with xor
     * he has lower start number,  0 if they are equal, 1 otherwise
     */
    @Override
    int compareTo( Contestant o);

    /**
     * Informs whether several object is equal to this contestant what means
     * it is a contestant and has the same start number
     * @param obj object to equal
     * @return <code>true</code> if the object is a contestant whith the same
     * start number, <code>false</code> otherwise
     */
    @Override
    boolean equals( Object obj);

    /**
     * Gives contestat's name
     * @return contestat's name
     */
    String getFullName();

    /**
     * Gives contestat's start nuber
     * @return contestat's start nuber
     */
    int getStartNumber();

    /**
     * Gives contestat's start time
     * @return contestat's start time
     */
    Date getStartTime();

    /**
     * Gives contestant's time -- the subtraction of finish time and start time
     * @return the subtraction of finish time and start time
     * @throws IllegalStateException if the performance has not been finished yet
     */
    Date getTime();

    /**
     * Informs whether contestant has allready started
     * @return true if contestant has started, false otherwise
     */
    boolean hasStarted();

    /**
     * Generates hash code of this object. The only atribute is start number
     *
     * @return hash code of this object
     */
    @Override
    int hashCode();

    /**
     * Gives the textual representation of the contestant
     * @return textual representation of the contestant (<full name> (No <start number>)
     */
    @Override
    String toString();

    /**
     * Start this contestant, the time begins to run
     * @param startTime date/time of start
     * @throws IllegalStateException if contestant allready started
     */
    void start( Date startTime);

    /**
     * Finish this contestant's performance, the time stops
     * @param startTime date/time of entering the finish
     * @throws IllegalStateException if contestant not started
     */
    void finish(Date finishTime);
}
