    /*
 * ContestantImpl.java
 *
 * Created on Pondelok, 2007, december 3, 14:40
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package cz.muni.fi.pb162.timetrial;

import java.util.Date;

/**
 * TimeTrial contestant identified by start number, described by name
 * 
 * @author misko
 */
public final class ContestantImpl implements Contestant {

    private final int startNumber;
    private final String fullName;
    private Date startTime;
    private Date time = null;

    /**
     * Creates new contestant identified by no described by name
     * @param startNumber start number of the contestant
     * @param fullName full name (name surname) of the contestant
     */
    public ContestantImpl(int no, String fullName) {
        this.startNumber = no;
        this.fullName = fullName;
    }

    /**
     * Informs whether several object is equal to this contestant what means
     * it is a contestant and has the same start number
     * @param obj object to equal
     * @return <code>true</code> if the object is a contestant whith the same 
     *      start number, <code>false</code> otherwise
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (!(this instanceof Contestant)) {
            return false;
        }
        final Contestant other = (Contestant) obj;
        if (this.startNumber != other.getStartNumber()) {
            return false;
        }
        return true;
    }

    /**
     * Generates hash code of this object. The only atribute is start number
     * 
     * @return hash code of this object
     */
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 13 * hash + this.startNumber;
        return hash;
    }

    /**
     * Compares this contestant with another one. Ordering is by (time,id)
     * lexicographicly. Object with n/a time are considered as the had infinite time
     * @param o contestant to compare
     * @return -1 if the this contestant is better than the compared with xor 
     * he has lower start number,  0 if they are equal, 1 otherwise
     */
    @Override
    public int compareTo(Contestant o) {
        if (equals(o)) {
            return 0;
        }
        if (time == null && o.getTime() == null) {
            return 0;
        }
        if (time == null) {
            return 1;
        }
        if (o.getTime() == null) {
            return -1;
        }
        if (time.equals(o.getTime())) {
            return (startNumber - o.getStartNumber());
        }

        return time.compareTo(o.getTime());

    }

    /**
     * Gives contestat's start nuber
     * @return contestat's start nuber
     */
    public int getStartNumber() {
        return startNumber;
    }

    /**
     * Gives contestat's name
     * @return contestat's name
     */
    public String getFullName() {
        return fullName;
    }

    /**
     * Informs whether contestant has allready started
     * @return true if contestant has started, false otherwise
     */
    public boolean hasStarted() {
        return startTime != null;
    }

    /**
     * Gives contestat's start time
     * @return contestat's start time
     */
    public Date getStartTime() {
        if (startTime == null) {
            throw new IllegalStateException("contestant not started");
        }
        return startTime;
    }

    /**
     * Start this contestant, the time begins to run
     * @param startTime date/time of start
     * @throws IllegalStateException if contestant allready started
     */
    public void start(Date startTime) {
        if (hasStarted()) {
            throw new IllegalStateException("contestant allready started");
        }
        this.startTime = startTime;
    }

    /**
     * Finish this contestant's performance, the time stops
     * @param startTime date/time of entering the finish
     * @throws IllegalStateException if contestant not started
     */
    public void finish(Date finishTime) {
        if (startTime == null) {
            throw new IllegalStateException("contestant not started");
        }
        this.time = new Date(finishTime.getTime() - startTime.getTime());
    }

    /**
     * Gives contestant's time -- the subtraction of finish time and start time
     * @return the subtraction of finish time and start time
     * @throws IllegalStateException if the performance has not been finished yet
     */
    public Date getTime() {
        if (time == null) {
            throw new IllegalStateException("contest not finished");
        }
        return time;
    }

    /**
     * Gives the textual representation of the contestant
     * @return textual representation of the contestant (<full name> (No <start number>)
     */
    @Override
    public String toString() {
        return fullName + " (No " + startNumber + ")";
    }
}
