/*
 * MainWindow.java
 *
 * Created on Utorok, 2008, december 9, 22:50
 */
package cz.muni.fi.pb162.timetrial;

import java.awt.Rectangle;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JScrollBar;

/**
 *
 * @author  misko
 */
public class MainWindow extends javax.swing.JFrame {

    private Timekeeping timekeeping = new Timekeeping();
    private TimeTable timeTableModel = new TimeTable((timekeeping));
    private List<Contestant> road = new LinkedList<Contestant>();

    private class Preparer extends Thread {

        public void run() {
            System.out.println("run");
            System.out.println(timekeeping.hasNext());
            Random random = new Random();
            while (timekeeping.hasNext()) {
                try {
                    synchronized (timekeeping) {
                        if (!timekeeping.hasNext()) {
                            continue;
                        }
                        final Contestant competitor = timekeeping.prepare();
                        log.append(competitor.toString() + " should prepare to start\n");
                        scrollLog();
                    }
                    synchronized (road) {
                        if (Math.random() > 0.8 && road.size() > 2) {
                            int first = random.nextInt(road.size() - 1);
                            road.add(first, road.get(first + 1));
                            road.remove(first + 2);
                        }
                    }
                    Thread.sleep(2000 + Math.round(Math.random() * 100));
                } catch (InterruptedException ex) {
                    Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private class Starter extends Thread {

        public void run() {
            try {
                Thread.sleep(5000);
                while (timekeeping.hasNext()) {
                    synchronized (timekeeping) {
                        if (!timekeeping.hasNext()) {
                            continue;
                        }
                        final Contestant competitor = timekeeping.prepare();
                        timekeeping.start();
                        road.add(competitor);
                        log.append("3, 2, 1, GO!, " + competitor.toString() + "\n");
                        scrollLog();
                    }
                    Thread.sleep(5000 + Math.round(Math.random() * +100));
                }
            } catch (InterruptedException ex) {
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private class Finisher extends Thread {

        public void run() {
            try {
                Thread.sleep(20000);
                while (!road.isEmpty()) {
                    synchronized (road) {
                        if (road.isEmpty()) {
                            continue;
                        }
                        final Contestant competitor = road.remove(0);
                        timekeeping.finish(competitor.getStartNumber());
                        log.append(competitor.toString() + " finished\n");
                        scrollLog();
                    }
                    Thread.sleep(5000 + Math.round(Math.random() * +2000 - 1000));
                }
            } catch (InterruptedException ex) {
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            }
            synchronized (timekeeping) {
                try {
                    timeTableModel.save("timetable.txt");
                } catch (IOException ex) {
                    log.append(ex.getMessage());
                }
            }

        }
    }

    private void scrollLog() {
        log.scrollRectToVisible(new Rectangle(0, log.getRows() * 200, 1, 1));
    }

    private class TableActualisation extends Thread {

        public void run() {
            try {
                while (true) {
                    synchronized (timekeeping) {
                        timeTable.setText(timeTableModel.toString());
                    }
                    Thread.sleep(100);
                }
            } catch (InterruptedException ex) {
                Logger.getLogger(MainWindow.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /** Creates new form MainWindow */
    public MainWindow() {
        try {

            initComponents();
            new AutoRegistration(timekeeping).load("competitors.txt");
//        timekeeping.register(new ContestantImpl(14, "Jan Ulrich"));
//        timekeeping.register(new ContestantImpl(151, "Lance Armstrong"));
//        timekeeping.register(new ContestantImpl(174, "Ivan Basso"));
//        timekeeping.register(new ContestantImpl(99, "David Miller"));
        } catch (FileNotFoundException ex) {
            log.append(ex.getMessage());
            return;
        } catch (IOException ex) {
            log.append(ex.getMessage());
            return;
        }
//        timekeeping.register(new ContestantImpl(14, "Jan Ulrich"));
//        timekeeping.register(new ContestantImpl(151, "Lance Armstrong"));
//        timekeeping.register(new ContestantImpl(174, "Ivan Basso"));
//        timekeeping.register(new ContestantImpl(99, "David Miller"));
        new Preparer().start();
        new Starter().start();
        new Finisher().start();
        new TableActualisation().start();

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        log = new javax.swing.JTextArea();
        jScrollPane2 = new javax.swing.JScrollPane();
        timeTable = new javax.swing.JTextArea();
        timeTableCaption = new javax.swing.JLabel();
        logCaption = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        log.setColumns(20);
        log.setRows(5);
        jScrollPane1.setViewportView(log);

        timeTable.setColumns(20);
        timeTable.setRows(5);
        jScrollPane2.setViewportView(timeTable);

        timeTableCaption.setText("Time Table");

        logCaption.setText("log");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(139, 139, 139)
                        .addComponent(timeTableCaption))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(12, 12, 12)
                                .addComponent(logCaption))
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 345, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 426, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(20, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(timeTableCaption)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(logCaption)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 119, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {

        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new MainWindow().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea log;
    private javax.swing.JLabel logCaption;
    private javax.swing.JTextArea timeTable;
    private javax.swing.JLabel timeTableCaption;
    // End of variables declaration//GEN-END:variables
}
